@@ -562,7 +562,7 @@ class KCAL_EXPORT Calendar : public QObject, public CustomProperties,
 
       @param start is the starting date.
       @param end is the ending date.
-      @param timespec time zone etc. to interpret @p start and @p end,
+      @param timeSpec time zone etc. to interpret @p start and @p end,
                       or the calendar's default time spec if none is specified
       @param inclusive if true only Events which are completely included
       within the date range are returned.
@@ -571,7 +571,7 @@ class KCAL_EXPORT Calendar : public QObject, public CustomProperties,
       date range.
     */
     Event::List events( const QDate &start, const QDate &end,
-                        const KDateTime::Spec &timespec = KDateTime::Spec(),
+                        const KDateTime::Spec &timeSpec = KDateTime::Spec(),
                         bool inclusive = false );
 
     /**
@@ -580,7 +580,7 @@ class KCAL_EXPORT Calendar : public QObject, public CustomProperties,
       @a sortDirection.
 
       @param date request filtered Event list for this QDate only.
-      @param timespec time zone etc. to interpret @p start and @p end,
+      @param timeSpec time zone etc. to interpret @p start and @p end,
                       or the calendar's default time spec if none is specified
       @param sortField specifies the EventSortField.
       @param sortDirection specifies the SortDirection.
@@ -589,7 +589,7 @@ class KCAL_EXPORT Calendar : public QObject, public CustomProperties,
     */
     Event::List events(
       const QDate &date,
-      const KDateTime::Spec &timespec = KDateTime::Spec(),
+      const KDateTime::Spec &timeSpec = KDateTime::Spec(),
       EventSortField sortField = EventSortUnsorted,
       SortDirection sortDirection = SortDirectionAscending );
 
@@ -621,7 +621,7 @@ class KCAL_EXPORT Calendar : public QObject, public CustomProperties,
 
       @param start is the starting date
       @param end is the ending date
-      @param timespec time zone etc. to interpret @p start and @p end,
+      @param timeSpec time zone etc. to interpret @p start and @p end,
                       or the calendar's default time spec if none is specified
       @param inclusive if true only Events which are completely included
       within the date range are returned.
@@ -630,7 +630,7 @@ class KCAL_EXPORT Calendar : public QObject, public CustomProperties,
       date range.
     */
     virtual Event::List rawEvents( const QDate &start, const QDate &end,
-                                   const KDateTime::Spec &timespec = KDateTime::Spec(),
+                                   const KDateTime::Spec &timeSpec = KDateTime::Spec(),
                                    bool inclusive = false ) = 0;
 
     /**
@@ -639,7 +639,7 @@ class KCAL_EXPORT Calendar : public QObject, public CustomProperties,
       @a sortDirection.
 
       @param date request unfiltered Event list for this QDate only
-      @param timespec time zone etc. to interpret @p date,
+      @param timeSpec time zone etc. to interpret @p date,
                       or the calendar's default time spec if none is specified
       @param sortField specifies the EventSortField
       @param sortDirection specifies the SortDirection
@@ -647,7 +647,7 @@ class KCAL_EXPORT Calendar : public QObject, public CustomProperties,
       @return the list of sorted, unfiltered Events occurring on @p date
     */
     virtual Event::List rawEventsForDate(
-      const QDate &date, const KDateTime::Spec &timespec = KDateTime::Spec(),
+      const QDate &date, const KDateTime::Spec &timeSpec = KDateTime::Spec(),
       EventSortField sortField = EventSortUnsorted,
       SortDirection sortDirection = SortDirectionAscending ) = 0;
 
@@ -850,6 +850,30 @@ class KCAL_EXPORT Calendar : public QObject, public CustomProperties,
     */
     virtual Journal *journal( const QString &uid ) = 0;
 
+    /**
+      Emits the beginBatchAdding() signal.
+
+      This should be called before adding a batch of incidences with
+      addIncidence( Incidence *), addTodo( Todo *), addEvent( Event *)
+      or addJournal( Journal *). Some Calendars are connected to this
+      signal, e.g: CalendarResources uses it to know a series of
+      incidenceAdds are related so the user isn't prompted multiple
+      times which resource to save the incidence to
+
+      @since 4.4
+    */
+    void beginBatchAdding();
+
+    /**
+      Emits the endBatchAdding() signal.
+
+      Used with beginBatchAdding(). Should be called after
+      adding all incidences.
+
+      @since 4.4
+    */
+    void endBatchAdding();
+
   // Relations Specific Methods //
 
     /**
@@ -874,7 +898,8 @@ class KCAL_EXPORT Calendar : public QObject, public CustomProperties,
       Sets the calendar filter.
 
       @param filter a pointer to a CalFilter object which will be
-      used to filter Calendar Incidences.
+      used to filter Calendar Incidences. The Calendar takes
+      ownership of @p filter.
 
       @see filter()
     */
@@ -969,6 +994,8 @@ class KCAL_EXPORT Calendar : public QObject, public CustomProperties,
      */
     void unregisterObserver( CalendarObserver *observer );
 
+    using QObject::event;   // prevent warning about hidden virtual method
+
   Q_SIGNALS:
     /**
       Signals that the calendar has been modified.
@@ -985,6 +1012,18 @@ class KCAL_EXPORT Calendar : public QObject, public CustomProperties,
      */
     void calendarLoaded();
 
+    /**
+      @see beginBatchAdding()
+      @since 4.4
+     */
+    void batchAddingBegins();
+
+    /**
+      @see endBatchAdding()
+      @since 4.4
+     */
+    void batchAddingEnds();
+
   protected:
     /**
       The Observer interface. So far not implemented.
