@@ -40,7 +40,7 @@
 #include <kdebug.h>
 
 #include <QtCore/QList>
-#include <QTextDocument> // for Qt::escape() and Qt::mightBeRichText()
+#include <QtGui/QTextDocument> // for Qt::escape() and Qt::mightBeRichText()
 
 using namespace KCal;
 
@@ -53,11 +53,17 @@ class KCal::Incidence::Private
 {
   public:
     Private()
-      : mRecurrence( 0 ),
+      : mDescriptionIsRich( false ),
+        mSummaryIsRich( false ),
+        mLocationIsRich( false ),
+        mRecurrence( 0 ),
         mStatus( StatusNone ),
         mSecrecy( SecrecyPublic ),
         mPriority( 0 ),
-        mRelatedTo( 0 )
+        mRelatedTo( 0 ),
+        mGeoLatitude( 0 ),
+        mGeoLongitude( 0 ),
+        mHasGeo( false )
     {
       mAlarms.setAutoDelete( true );
       mAttachments.setAutoDelete( true );
@@ -73,17 +79,18 @@ class KCal::Incidence::Private
         mLocation( p.mLocation ),
         mLocationIsRich( p.mLocationIsRich ),
         mCategories( p.mCategories ),
+        mRecurrence( p.mRecurrence ),
         mResources( p.mResources ),
         mStatus( p.mStatus ),
         mStatusString( p.mStatusString ),
         mSecrecy( p.mSecrecy ),
         mPriority( p.mPriority ),
         mSchedulingID( p.mSchedulingID ),
-        mRelatedTo( 0 ),
-        mRelatedToUid( p.mRelatedToUid )
-// TODO: reenable attributes currently commented out.
-//  Incidence *mRelatedTo;          Incidence *mRelatedTo;
-//  Incidence::List mRelations;    Incidence::List mRelations;
+        mRelatedTo( p.mRelatedTo ),
+        mRelatedToUid( p.mRelatedToUid ),
+        mGeoLatitude( p.mGeoLatitude ),
+        mGeoLongitude( p.mGeoLongitude ),
+        mHasGeo( p.mHasGeo )
     {
       mAlarms.setAutoDelete( true );
       mAttachments.setAutoDelete( true );
@@ -118,7 +125,10 @@ class KCal::Incidence::Private
 
     Incidence *mRelatedTo;           // incidence this is related to
     QString mRelatedToUid;           // incidence (by Uid) this is related to
-    Incidence::List mRelations;      // a list of incidences this is related to
+    Incidence::List mRelations;      // a list of incidences related to this
+    float mGeoLatitude;              // Specifies latitude in decimal degrees
+    float mGeoLongitude;             // Specifies longitude in decimal degrees
+    bool mHasGeo;                    // if incidence has geo data
 };
 //@endcond
 
@@ -138,36 +148,35 @@ Incidence::Incidence( const Incidence &i )
 
 void Incidence::init( const Incidence &i )
 {
-// TODO: reenable attributes currently commented out.
   d->mRevision = i.d->mRevision;
   d->mCreated = i.d->mCreated;
   d->mDescription = i.d->mDescription;
   d->mSummary = i.d->mSummary;
   d->mCategories = i.d->mCategories;
-//  Incidence *mRelatedTo;          Incidence *mRelatedTo;
-  d->mRelatedTo = 0;
+  d->mRelatedTo = i.d->mRelatedTo;
   d->mRelatedToUid = i.d->mRelatedToUid;
-//  Incidence::List mRelations;    Incidence::List mRelations;
+  d->mRelations = i.d->mRelations;
   d->mResources = i.d->mResources;
   d->mStatusString = i.d->mStatusString;
   d->mStatus = i.d->mStatus;
   d->mSecrecy = i.d->mSecrecy;
   d->mPriority = i.d->mPriority;
   d->mLocation = i.d->mLocation;
+  d->mGeoLatitude = i.d->mGeoLatitude;
+  d->mGeoLongitude = i.d->mGeoLongitude;
+  d->mHasGeo = i.d->mHasGeo;
 
   // Alarms and Attachments are stored in ListBase<...>, which is a QValueList<...*>.
   // We need to really duplicate the objects stored therein, otherwise deleting
   // i will also delete all attachments from this object (setAutoDelete...)
-  Alarm::List::ConstIterator it;
-  for ( it = i.d->mAlarms.begin(); it != i.d->mAlarms.end(); ++it ) {
-    Alarm *b = new Alarm( **it );
+  foreach ( Alarm *alarm, i.d->mAlarms ) {
+    Alarm *b = new Alarm( *alarm );
     b->setParent( this );
     d->mAlarms.append( b );
   }
 
-  Attachment::List::ConstIterator it1;
-  for ( it1 = i.d->mAttachments.begin(); it1 != i.d->mAttachments.end(); ++it1 ) {
-    Attachment *a = new Attachment( **it1 );
+  foreach ( Attachment *attachment, i.d->mAttachments ) {
+    Attachment *a = new Attachment( *attachment );
     d->mAttachments.append( a );
   }
 
@@ -181,14 +190,16 @@ void Incidence::init( const Incidence &i )
 
 Incidence::~Incidence()
 {
-  Incidence::List Relations = d->mRelations;
-  List::ConstIterator it;
-  for ( it = Relations.begin(); it != Relations.end(); ++it ) {
-    if ( (*it)->relatedTo() == this ) {
-      (*it)->d->mRelatedTo = 0;
+  Incidence::List relations = d->mRelations;
+  foreach ( Incidence *incidence, relations ) {
+    if ( incidence->relatedTo() == this ) {
+      incidence->setRelatedTo( 0 );
     }
   }
 
+  if ( relatedTo() ) {
+    relatedTo()->removeRelation( this );
+  }
   delete d->mRecurrence;
   delete d;
 }
@@ -203,6 +214,11 @@ static bool stringCompare( const QString &s1, const QString &s2 )
 //@endcond
 Incidence &Incidence::operator=( const Incidence &other )
 {
+  // check for self assignment
+  if ( &other == this ) {
+    return *this;
+  }
+
   d->clear();
   //TODO: should relations be cleared out, as in destructor???
   IncidenceBase::operator=( other );
@@ -216,9 +232,11 @@ bool Incidence::operator==( const Incidence &i2 ) const
     return false; // no need to check further
   }
 
-  Alarm::List::ConstIterator a1 = alarms().begin();
+  Alarm::List::ConstIterator a1 = alarms().constBegin();
+  Alarm::List::ConstIterator a1end = alarms().constEnd();
   Alarm::List::ConstIterator a2 = i2.alarms().begin();
-  for ( ; a1 != alarms().end() && a2 != i2.alarms().end(); ++a1, ++a2 ) {
+  Alarm::List::ConstIterator a2end = i2.alarms().constEnd();
+  for ( ; a1 != a1end && a2 != a2end; ++a1, ++a2 ) {
     if ( **a1 == **a2 ) {
       continue;
     } else {
@@ -373,7 +391,7 @@ QString Incidence::richDescription() const
   if ( descriptionIsRich() ) {
     return d->mDescription;
   } else {
-    return Qt::escape( d->mDescription ).replace( "\n", "<br/>" );
+    return Qt::escape( d->mDescription ).replace( '\n', "<br/>" );
   }
 }
 
@@ -407,7 +425,7 @@ QString Incidence::richSummary() const
   if ( summaryIsRich() ) {
     return d->mSummary;
   } else {
-    return Qt::escape( d->mSummary ).replace( "\n", "<br/>" );
+    return Qt::escape( d->mSummary ).replace( '\n', "<br/>" );
   }
 }
 
@@ -436,7 +454,7 @@ void Incidence::setCategories( const QString &catStr )
     return;
   }
 
-  d->mCategories = catStr.split( "," );
+  d->mCategories = catStr.split( ',' );
 
   QStringList::Iterator it;
   for ( it = d->mCategories.begin();it != d->mCategories.end(); ++it ) {
@@ -458,7 +476,7 @@ QString Incidence::categoriesStr() const
 
 void Incidence::setRelatedToUid( const QString &relatedToUid )
 {
-  if ( mReadOnly || d->mRelatedToUid == relatedToUid ) {
+  if ( d->mRelatedToUid == relatedToUid ) {
     return;
   }
   d->mRelatedToUid = relatedToUid;
@@ -472,7 +490,7 @@ QString Incidence::relatedToUid() const
 
 void Incidence::setRelatedTo( Incidence *incidence )
 {
-  if ( mReadOnly || d->mRelatedTo == incidence ) {
+  if ( d->mRelatedTo == incidence ) {
     return;
   }
   if ( d->mRelatedTo ) {
@@ -509,7 +527,9 @@ void Incidence::addRelation( Incidence *incidence )
 void Incidence::removeRelation( Incidence *incidence )
 {
   d->mRelations.removeRef( incidence );
-  d->mRelatedToUid.clear();
+  if ( d->mRelatedToUid == incidence->uid() ) {
+    d->mRelatedToUid.clear();
+  }
 //  if (incidence->getRelatedTo() == this) incidence->setRelatedTo(0);
 }
 
@@ -592,8 +612,8 @@ QList<KDateTime> Incidence::startDateTimesForDate( const QDate &date,
   while ( tmpday <= date ) {
     if ( recurrence()->recursOn( tmpday, timeSpec ) ) {
       QList<QTime> times = recurrence()->recurTimesOn( tmpday, timeSpec );
-      for ( QList<QTime>::ConstIterator it = times.begin(); it != times.end(); ++it ) {
-        tmp = KDateTime( tmpday, *it, start.timeSpec() );
+      foreach ( const QTime &time, times ) {
+        tmp = KDateTime( tmpday, time, start.timeSpec() );
         if ( endDateForStart( tmp ) >= kdate ) {
           result << tmp;
         }
@@ -632,8 +652,8 @@ QList<KDateTime> Incidence::startDateTimesForDateTime( const KDateTime &datetime
     if ( recurrence()->recursOn( tmpday, datetime.timeSpec() ) ) {
       // Get the times during the day (in start date's time zone) when recurrences happen
       QList<QTime> times = recurrence()->recurTimesOn( tmpday, start.timeSpec() );
-      for ( QList<QTime>::ConstIterator it = times.begin(); it != times.end(); ++it ) {
-        tmp = KDateTime( tmpday, *it, start.timeSpec() );
+      foreach ( const QTime &time, times ) {
+        tmp = KDateTime( tmpday, time, start.timeSpec() );
         if ( !( tmp > datetime || endDateForStart( tmp ) < datetime ) ) {
           result << tmp;
         }
@@ -694,9 +714,9 @@ Attachment::List Incidence::attachments( const QString &mime ) const
 {
   Attachment::List attachments;
   Attachment::List::ConstIterator it;
-  for ( it = d->mAttachments.begin(); it != d->mAttachments.end(); ++it ) {
-    if ( (*it)->mimeType() == mime ) {
-      attachments.append( *it );
+  foreach ( Attachment *attachment, d->mAttachments ) {
+    if ( attachment->mimeType() == mime ) {
+      attachments.append( attachment );
     }
   }
   return attachments;
@@ -875,9 +895,8 @@ void Incidence::clearAlarms()
 
 bool Incidence::isAlarmEnabled() const
 {
-  Alarm::List::ConstIterator it;
-  for ( it = d->mAlarms.begin(); it != d->mAlarms.end(); ++it ) {
-    if ( (*it)->enabled() ) {
+  foreach ( Alarm *alarm, d->mAlarms ) {
+    if ( alarm->enabled() ) {
       return true;
     }
   }
@@ -910,7 +929,7 @@ QString Incidence::richLocation() const
   if ( locationIsRich() ) {
     return d->mLocation;
   } else {
-    return Qt::escape( d->mLocation ).replace( "\n", "<br/>" );
+    return Qt::escape( d->mLocation ).replace( '\n', "<br/>" );
   }
 }
 
@@ -933,6 +952,51 @@ QString Incidence::schedulingID() const
   return d->mSchedulingID;
 }
 
+bool Incidence::hasGeo() const
+{
+  return d->mHasGeo;
+}
+
+void Incidence::setHasGeo( bool hasGeo )
+{
+  if ( mReadOnly ) {
+    return;
+  }
+
+  d->mHasGeo = hasGeo;
+  updated();
+}
+
+float &Incidence::geoLatitude() const
+{
+  return d->mGeoLatitude;
+}
+
+void Incidence::setGeoLatitude( float geolatitude )
+{
+  if ( mReadOnly ) {
+    return;
+  }
+
+  d->mGeoLatitude = geolatitude;
+  updated();
+}
+
+float &Incidence::geoLongitude() const
+{
+  return d->mGeoLongitude;
+}
+
+void Incidence::setGeoLongitude( float geolongitude )
+{
+  if ( mReadOnly ) {
+    return;
+  }
+
+  d->mGeoLongitude = geolongitude;
+  updated();
+}
+
 /** Observer interface for the recurrence class. If the recurrence is changed,
     this method will be called for the incidence the recurrence object
     belongs to. */
