@@ -129,7 +129,7 @@ void CalendarLocalTest::testIncidences()
   Todo *todo2 = new Todo();
   todo2->setUid( "4" );
   todo2->setDtStart( KDateTime( dt ).addDays( 1 ) );
-  todo2->setFloats( true );
+  todo2->setAllDay( true );
   todo2->setSummary( "<qt><h1>Todo2 Summary</h1></qt>", true );
   todo2->setDescription( "This is a description of a todo" );
   todo2->setLocation( "<html><a href=\"http://www.fred.com\">this place</a></html>", true );
@@ -148,3 +148,37 @@ void CalendarLocalTest::testIncidences()
   cal.close();
   unlink( "foo.ics" );
 }
+
+void CalendarLocalTest::testRelationsCrash()
+{
+  // Before, there was a crash that occured only when reloading a calendar in which
+  // the incidences had special relations.
+  // This test tests that scenario, and will crash if it fails.
+  CalendarLocal cal( KDateTime::UTC );
+  cal.load( ICALTESTDATADIR "test_relations.ics" );
+  Todo::List oldTodos = cal.todos();
+  cal.load( ICALTESTDATADIR "test_relations.ics" );
+  Todo::List newTodos = cal.todos();
+
+  // We can saftely access the old deleted todos here, since they are not really deleted
+  // and are still kept in a map of deleted items somewhere.
+  //
+  // Here we make sure that non of the old items have connections to the new items, and
+  // the other way around.
+  foreach ( Todo * oldTodo, oldTodos ) {
+    foreach ( Todo *newTodo, newTodos ) {
+      QVERIFY( oldTodo != newTodo );
+
+      // Make sure that none of the new todos point to an old, deleted todo
+      QVERIFY( newTodo->relatedTo() != oldTodo );
+      QVERIFY( !newTodo->relations().contains( oldTodo ) );
+
+      // Make sure that none of the old todos point to a new todo
+      QVERIFY( oldTodo->relatedTo() != newTodo );
+      QVERIFY( !oldTodo->relations().contains( newTodo ) );
+    }
+  }
+
+  cal.close();
+}
+
