@@ -18,6 +18,8 @@
 */
 
 #include "headertest.h"
+
+#include <KDebug>
 #include <qtest_kde.h>
 
 #include <kmime_headers.h>
@@ -74,6 +76,15 @@ void HeaderTest::testIdentHeader()
 
   // assemble the final header
   QCOMPARE( h->as7BitString( false ), QByteArray("<1234@local.machine.example> <3456@example.net> <abcd.1234@local.machine.tld> <78910@example.net>") );
+  delete h;
+
+  // parsing of ident with literal domain
+  h = new Headers::Generics::Ident();
+  const QByteArray ident = QByteArray( "<O55F3Y9E5MmKFwBN@[127.0.0.1]>" );
+  h->appendIdentifier( ident );
+  QEXPECT_FAIL( "", "Parsing strips angle brackets.", Continue );
+  QCOMPARE( h->as7BitString( false ), QByteArray( ident ) );
+  delete h;
 }
 
 void HeaderTest::testAddressListHeader()
@@ -222,10 +233,14 @@ void HeaderTest::testAddressListHeader()
   QCOMPARE( h->as7BitString( false ), QByteArray("\"first name (nickname)\" <first.name@domain.tld>") );
   delete h;
 
-  // rfc 2047 encoding in quoted name (which is not allowed there)
+  // rfc 2047 encoding in quoted name (it is not allowed there as per the RFC, but it happens)
   h = new Headers::Generics::AddressList();
   h->from7BitString( QByteArray( "\"Ingo =?iso-8859-15?q?Kl=F6cker?=\" <kloecker@kde.org>" ) );
   QCOMPARE( h->mailboxes().count(), 1 );
+  QEXPECT_FAIL( "", "RFC2047-encoded words in quoted string are not parsed (conformant with the RFC, but not with some software)", Continue );
+  // some software == current KMail (v1.12.90) ...
+  QCOMPARE( h->asUnicodeString(), QString::fromUtf8( "\"Ingo Klöcker\" <kloecker@kde.org>" ) );
+  // The following test is the "conformant" version of the test (kept to catch regression as long as the previous test doesn't pass).
   QCOMPARE( h->asUnicodeString(), QString::fromUtf8( "Ingo =?iso-8859-15?q?Kl=F6cker?= <kloecker@kde.org>" ) );
   delete h;
 
@@ -333,6 +348,12 @@ void HeaderTest::testMailCopiesToHeader()
   QVERIFY( h->alwaysCopy() );
   delete h;
 
+  h = new MailCopiesTo( 0, "poster" );
+  QVERIFY( h->addresses().isEmpty() );
+  QVERIFY( !h->isEmpty() );
+  QVERIFY( h->alwaysCopy() );
+  delete h;
+
   // parse never copy
   h = new MailCopiesTo( 0, "never" );
   QVERIFY( h->addresses().isEmpty() );
@@ -340,6 +361,17 @@ void HeaderTest::testMailCopiesToHeader()
   QVERIFY( h->neverCopy() );
   delete h;
 
+  h = new MailCopiesTo( 0, "nobody" );
+  QVERIFY( h->addresses().isEmpty() );
+  QVERIFY( !h->isEmpty() );
+  QVERIFY( h->neverCopy() );
+  delete h;
+
+  // parsing is case-insensitive
+  h = new MailCopiesTo( 0, "AlWays" );
+  QVERIFY( h->alwaysCopy() );
+  delete h;
+
   // parse address
   h = new MailCopiesTo( 0, "vkrause@kde.org" );
   QVERIFY( !h->addresses().isEmpty() );
@@ -465,6 +497,12 @@ void HeaderTest::testContentTypeHeader()
   QVERIFY( h->isPlainText() );
   QCOMPARE( h->name(), QString( "PIN_Brief_box1@xx.xxx.censored_Konfigkarte.configuration.txt" ) );
   delete h;
+
+  // bug #197958 (name of Content-Type sent by Mozilla Thunderbird are not parsed -- test case generated with v2.0.0.22)
+  h = new ContentType( 0, "text/plain;\n name=\"=?ISO-8859-1?Q?lor=E9m_ipsum=2Etxt?=\"" );
+  QEXPECT_FAIL( "", "Name of attachment send by Mozilla Thunderbird are not parsed.", Continue );
+  QCOMPARE( h->name(), QString::fromUtf8( "lorém ipsum.txt" ) );
+  delete h;
 }
 
 void HeaderTest::testTokenHeader()
@@ -609,6 +647,22 @@ void HeaderTest::testDateHeader()
   QCOMPARE( h->dateTime().time(), QTime( 9, 55, 6 ) );
   QCOMPARE( h->dateTime().utcOffset(), -6 * 3600 );
   delete h;
+
+  // Make sure uppercase OCT is parsed correctly - bug 150620
+  h = new Date( 0, "08 OCT 08 16:54:05 +0000" );
+  QVERIFY( !h->isEmpty() );
+  QCOMPARE( h->dateTime().date(), QDate( 2008, 10, 8 ) );
+  QCOMPARE( h->dateTime().time(), QTime( 16, 54, 05 ) );
+  QCOMPARE( h->dateTime().utcOffset(), 0 );
+  delete h;
+
+  // Test for bug 207766
+  h = new Date( 0, "Fri, 18 Sep 2009 04:44:55 -0400" );
+  QVERIFY( !h->isEmpty() );
+  QCOMPARE( h->dateTime().date(), QDate( 2009, 9, 18 ) );
+  QCOMPARE( h->dateTime().time(), QTime( 4, 44, 55 ) );
+  QCOMPARE( h->dateTime().utcOffset(), -4 * 3600 );
+  delete h;
 }
 
 void HeaderTest::testLinesHeader()
@@ -743,4 +797,36 @@ void HeaderTest::noAbstractHeaders()
   UserAgent* h24 = new UserAgent(); delete h24;
 }
 
+void HeaderTest::testInvalidButOkQEncoding()
+{
+  // A stray '?' should not confuse the parser
+  Subject subject;
+  subject.from7BitString( "=?us-ascii?q?Why?_Why_do_some_clients_violate_the_RFC?" "?=" );
+  QCOMPARE( subject.as7BitString( false ), QByteArray( "Why? Why do some clients violate the RFC?" ) );
+}
+
+void HeaderTest::testInvalidQEncoding_data()
+{
+  QTest::addColumn<QString>("encodedWord");
+
+  // All examples below should not be treated as invalid encoded strings, since the '?=' is missing
+  QTest::newRow("") << QString( "=?us-ascii?q?Why?_Why_do_some_clients_violate_the_RFC??" );
+  QTest::newRow("") << QString( "=?us-ascii?q?Why?_Why_do_some_clients_violate_the_RFC?" );
+  QTest::newRow("") << QString( "=?us-ascii?q?Why?_Why_do_some_clients_violate_the_RFC" );
+}
+
+void HeaderTest::testInvalidQEncoding()
+{
+  using namespace HeaderParsing;
+  QFETCH( QString,encodedWord );
+
+  const char *data = encodedWord.toAscii().data();
+  const char *start = data + 1;
+  const char *end = data + strlen( data );
+  QString result;
+  QByteArray language;
+  QByteArray usedCS;
+  QVERIFY( !parseEncodedWord( start, end, result, language, usedCS ) );
+}
+
 #include "headertest.moc"
