@@ -61,7 +61,7 @@ class KPIMUtils::LinkLocator::Private
 //@endcond
 
 #if KDE_IS_VERSION( 4, 0, 95 )
-// Use a static for this as calls to KEmoticons::theme() are expensive
+// Use a static for this as calls to the KEmoticons constructor are expensive.
 K_GLOBAL_STATIC( KEmoticons, sEmoticons )
 #endif
 
@@ -107,23 +107,53 @@ QString LinkLocator::getUrl()
 {
   QString url;
   if ( atUrl() ) {
-    // handle cases like this: <link>http://foobar.org/</link>
+    // NOTE: see http://tools.ietf.org/html/rfc3986#appendix-A and especially appendix-C
+    // Appendix-C mainly says, that when extracting URLs from plain text, line breaks shall
+    // be allowed and should be ignored when the URI is extracted.
+
+    // This implementation follows this recommendation and
+    // allows the URL to be enclosed within different kind of brackets/quotes
+    // If an URL is enclosed, whitespace characters are allowed and removed, otherwise
+    // the URL ends with the first whitespace
+    // Also, if the URL is enclosed in brackets, the URL itself is not allowed
+    // to contain the closing bracket, as this would be detected as the end of the URL
+
+    QChar beforeUrl, afterUrl;
+
+    // detect if the url has been surrounded by brackets or quotes
+    if ( mPos > 0 ) {
+      beforeUrl = mText[mPos - 1];
+
+      if ( beforeUrl == '(' ) {
+        afterUrl = ')';
+      } else if ( beforeUrl == '[' ) {
+        afterUrl = ']';
+      } else if ( beforeUrl == '<' ) {
+        afterUrl = '>';
+      } else if ( beforeUrl == '>' ) { // for e.g. <link>http://.....</link>
+        afterUrl = '<';
+      } else if ( beforeUrl == '"' ) {
+        afterUrl = '"';
+      }
+    }
+
+    url.reserve( maxUrlLen() );  // avoid allocs
     int start = mPos;
-    while ( mPos < (int)mText.length() &&
-            mText[mPos] > ' ' && mText[mPos] != '"' &&
-            QString( "<>()[]" ).indexOf( mText[mPos] ) == -1 ) {
-      ++mPos;
+    while ( ( mPos < (int)mText.length() ) &&
+            ( mText[mPos].isPrint() || mText[mPos].isSpace() ) &&
+            ( ( afterUrl.isNull() && !mText[mPos].isSpace() ) ||
+              ( !afterUrl.isNull() && mText[mPos] != afterUrl ) ) ) {
+      if ( !mText[mPos].isSpace() ) {   // skip whitespace
+        url.append( mText[mPos] );
+        if ( url.length() > maxUrlLen() ) {
+          break;
+        }
+      }
+
+      mPos++;
     }
 
-    /* some URLs really end with:  # / & - _    */
-    const QString allowedSpecialChars = QString( "#/&-_" );
-    while ( mPos > start && mText[mPos-1].isPunct() &&
-            allowedSpecialChars.indexOf( mText[mPos-1] ) == -1 ) {
-      --mPos;
-    }
-
-    url = mText.mid( start, mPos - start );
-    if ( isEmptyUrl(url) || mPos - start > maxUrlLen() ) {
+    if ( isEmptyUrl(url) || ( url.length() > maxUrlLen() ) ) {
       mPos = start;
       url = "";
     } else {
@@ -268,20 +298,37 @@ QString LinkLocator::convertToHtml( const QString &plainText, int flags,
     ch = locator.mText[locator.mPos];
     if ( flags & PreserveSpaces ) {
       if ( ch == ' ' ) {
-        if ( startOfLine ) {
+        if ( locator.mPos + 1 < locator.mText.length() ) {
+          if ( locator.mText[locator.mPos + 1] != ' ' ) {
+
+            // A single space, make it breaking if not at the start or end of the line
+            const bool endOfLine = locator.mText[locator.mPos + 1] == '\n';
+            if ( !startOfLine && !endOfLine ) {
+              result += ' ';
+            } else {
+              result += "&nbsp;";
+            }
+          } else {
+
+            // Whitespace of more than one space, make it all non-breaking
+            while ( locator.mPos < locator.mText.length() && locator.mText[locator.mPos] == ' ' ) {
+              result += "&nbsp;";
+              locator.mPos++;
+              x++;
+            }
+
+            // We incremented once to often, undo that
+            locator.mPos--;
+            x--;
+          }
+        } else {
+          // Last space in the text, it is non-breaking
           result += "&nbsp;";
-          locator.mPos++, x++;
+        }
+
+        if ( startOfLine ) {
           startOfLine = false;
         }
-        while ( locator.mText[locator.mPos] == ' ' ) {
-          result += ' ';
-          locator.mPos++, x++;
-          if ( locator.mText[locator.mPos] == ' ' ) {
-            result += "&nbsp;";
-            locator.mPos++, x++;
-          }
-        }
-        locator.mPos--, x--;
         continue;
       } else if ( ch == '\t' ) {
         do
@@ -325,8 +372,7 @@ QString LinkLocator::convertToHtml( const QString &plainText, int flags,
             hyperlink = str;
           }
 
-          str = str.replace( '&', "&amp;" );
-          result += "<a href=\"" + hyperlink + "\">" + str + "</a>";
+          result += "<a href=\"" + hyperlink + "\">" + Qt::escape( str ) + "</a>";
           x += locator.mPos - start;
           continue;
         }
@@ -367,9 +413,13 @@ QString LinkLocator::convertToHtml( const QString &plainText, int flags,
     exclude << "(I)" << "(i)" << "(L)" << "(l)" << "(8)" << "(T)" << "(t)" << "(G)";
     exclude << "(g)" << "(F)" << "(f)" << "(H)";
     exclude << "8)" << "(N)" << "(n)" << "(Y)" << "(y)" << "(U)" << "(u)" << "(W)" << "(w)";
-    result = sEmoticons->theme().parseEmoticons( result, KEmoticonsTheme::StrictParse |
-                                                         KEmoticonsTheme::SkipHTML,
-                                                 exclude );
+    static QString cachedEmoticonsThemeName;
+    if ( cachedEmoticonsThemeName.isEmpty() ) {
+      cachedEmoticonsThemeName = KEmoticons::currentThemeName();
+    }
+    result =
+      sEmoticons->theme( cachedEmoticonsThemeName ).parseEmoticons(
+        result, KEmoticonsTheme::StrictParse | KEmoticonsTheme::SkipHTML, exclude );
   }
 #endif
 
@@ -405,7 +455,8 @@ QString LinkLocator::highlightedText()
   }
 
   QRegExp re =
-    QRegExp( QString( "\\%1([0-9A-Za-z]+)\\%2" ).arg( ch ).arg( ch ) );
+    QRegExp( QString( "\\%1((\\w+)([\\s-']\\w+)*( ?[,.:\\?!;])?)\\%2" ).arg( ch ).arg( ch ) );
+  re.setMinimal(true);
   if ( re.indexIn( mText, mPos ) == mPos ) {
     int length = re.matchedLength();
     // there must be a whitespace after the closing formating symbol
