@@ -18,16 +18,22 @@
 */
 
 #include "transport.h"
-#include "transportmanager.h"
-#include "mailtransport_defs.h"
 #include "legacydecrypt.h"
+#include "mailtransport_defs.h"
+#include "transportmanager.h"
+#include "transporttype_p.h"
 
-#include <kdebug.h>
-#include <klocale.h>
-#include <kmessagebox.h>
-#include <kstringhandler.h>
-#include <kwallet.h>
-#include <kconfiggroup.h>
+#include <QTimer>
+
+#include <KConfigGroup>
+#include <KDebug>
+#include <KLocalizedString>
+#include <KMessageBox>
+#include <KStringHandler>
+#include <KWallet/Wallet>
+
+#include <akonadi/agentinstance.h>
+#include <akonadi/agentmanager.h>
 
 using namespace MailTransport;
 using namespace KWallet;
@@ -39,6 +45,7 @@ using namespace KWallet;
 class TransportPrivate
 {
   public:
+    TransportType transportType;
     QString password;
     bool passwordLoaded;
     bool passwordDirty;
@@ -88,6 +95,40 @@ void Transport::setPassword( const QString &passwd )
   d->password = passwd;
 }
 
+void Transport::forceUniqueName()
+{
+  QStringList existingNames;
+  foreach ( Transport *t, TransportManager::self()->transports() ) {
+    if ( t->id() != id() ) {
+      existingNames << t->name();
+    }
+  }
+  int suffix = 1;
+  QString origName = name();
+  while ( existingNames.contains( name() ) ) {
+    setName( i18nc( "%1: name; %2: number appended to it to make "
+                    "it unique among a list of names", "%1 #%2", origName, suffix ) );
+    ++suffix;
+  }
+
+}
+
+void Transport::updatePasswordState()
+{
+  Transport *original = TransportManager::self()->transportById( id(), false );
+  if ( original == this ) {
+    kWarning() << "Tried to update password state of non-cloned transport.";
+    return;
+  }
+  if ( original ) {
+    d->password = original->d->password;
+    d->passwordLoaded = original->d->passwordLoaded;
+    d->passwordDirty = original->d->passwordDirty;
+  } else {
+    kWarning() << "Transport with this ID not managed by transport manager.";
+  }
+}
+
 bool Transport::isComplete() const
 {
   return !requiresAuthentication() || !storePassword() || d->passwordLoaded;
@@ -116,10 +157,39 @@ QString Transport::authenticationTypeString() const
 void Transport::usrReadConfig()
 {
   TransportBase::usrReadConfig();
+
+  setHost( host().trimmed() );
+
   if ( d->oldName.isEmpty() ) {
     d->oldName = name();
   }
 
+  // Set TransportType.
+  {
+    using namespace Akonadi;
+    d->transportType = TransportType();
+    d->transportType.d->mType = type();
+    kDebug() << "type" << type();
+    if( type() == EnumType::Akonadi ) {
+      const AgentInstance instance = AgentManager::self()->instance( host() );
+      if( !instance.isValid() ) {
+        kWarning() << "Akonadi transport with invalid resource instance.";
+      }
+      d->transportType.d->mAgentType = instance.type();
+      kDebug() << "agent type" << instance.type().name() << "id" << instance.type().identifier();
+    }
+    // Now we have the type and possibly agentType.  Get the name, description
+    // etc. from TransportManager.
+    const TransportType::List &types = TransportManager::self()->types();
+    int index = types.indexOf( d->transportType );
+    if( index != -1 ) {
+      d->transportType = types[ index ];
+    } else {
+      kWarning() << "Type unknown to manager.";
+      d->transportType.d->mName = i18nc( "An unknown transport type", "Unknown" );
+    }
+  }
+
   // we have everything we need
   if ( !storePassword() || d->passwordLoaded ) {
     return;
@@ -145,7 +215,11 @@ void Transport::usrReadConfig()
   } else {
     // read password if wallet is open, defer otherwise
     if ( Wallet::isOpen( Wallet::NetworkWallet() ) ) {
-      readPassword();
+      // Don't read the password right away because this can lead
+      // to reentrancy problems in KDBusServiceStarter when an application
+      // run in Kontact creates the transports (due to a QEventLoop in the
+      // synchronous KWallet openWallet call).
+      QTimer::singleShot( 0, this, SLOT(readPassword()) );
     }
   }
 }
@@ -235,6 +309,8 @@ void Transport::migrateToWallet()
   d->needsWalletMigration = false;
   KConfigGroup group( config(), currentGroup() );
   group.deleteEntry( "password" );
+  group.deleteEntry( "password-kmail" );
+  group.deleteEntry( "password-knode" );
   d->passwordDirty = true;
   d->storePasswordInFile = false;
   writeConfig();
@@ -245,3 +321,20 @@ Transport *Transport::clone() const
   QString id = currentGroup().mid( 10 );
   return new Transport( id );
 }
+
+TransportType Transport::transportType() const
+{
+  if( !d->transportType.isValid() ) {
+    kWarning() << "Invalid transport type.";
+  }
+  return d->transportType;
+}
+
+void Transport::setTransportType( const TransportType &type )
+{
+  Q_ASSERT( type.isValid() );
+  d->transportType = type;
+  setType( type.type() );
+}
+
+#include "transport.moc"
