@@ -1,7 +1,7 @@
 /*
  * This file is part of the syndication library
  *
- * Copyright (C) 2006 Frank Osterfeld <frank.osterfeld@kdemail.net>
+ * Copyright (C) 2006 Frank Osterfeld <osterfeld@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -31,7 +31,7 @@ namespace RDF {
 
 class Resource;
 //@cond PRIVATE
-typedef SharedPtr<Resource> ResourcePtr;
+typedef boost::shared_ptr<Resource> ResourcePtr;
 //@endcond
 
 /**
