@@ -1,7 +1,7 @@
 /*
     This file is part of LibSyndication.
 
-    Copyright (C) 2005 Frank Osterfeld <frank.osterfeld@kdemail.net>
+    Copyright (C) 2005 Frank Osterfeld <osterfeld@kde.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,22 +37,16 @@
 #include <klocale.h>
 #include <kurl.h>
 
-#include <QByteArray>
-#include <QDir>
-#include <QFile>
-#include <QString>
+#include <QtCore/QByteArray>
+#include <QtCore/QDir>
+#include <QtCore/QFile>
+#include <QtCore/QString>
 
 #include <cstdlib>
 #include <iostream>
 
 using namespace Syndication;
 
-static const KCmdLineOptions options[] =
-{
-    { "+url", I18N_NOOP("URL of feed"), 0 },
-    KCmdLineLastOption
-};
-
 TestLibSyndication::TestLibSyndication(const QString& url)
 {
     KUrl kurl;
@@ -72,6 +66,9 @@ void TestLibSyndication::slotLoadingComplete(Syndication::Loader* loader,
                     Syndication::FeedPtr feed,
                     Syndication::ErrorCode error)
 {
+    Q_UNUSED(loader)
+    Q_UNUSED(error)
+
     if (feed)
     {
         std::cout << feed->debugInfo().toUtf8().data() << std::endl;
@@ -93,8 +90,11 @@ int main(int argc, char **argv)
         return 1;
     }
 
-    KAboutData aboutData("testlibsyndication", "testlibsyndication", "0.1");
+    KAboutData aboutData("testlibsyndication", 0, ki18n("testlibsyndication"), "0.1");
     KCmdLineArgs::init(argc, argv, &aboutData);
+
+    KCmdLineOptions options;
+    options.add("+url", ki18n("URL of feed"));
     KCmdLineArgs::addCmdLineOptions(options);
     KApplication app;
 
@@ -102,6 +102,7 @@ int main(int argc, char **argv)
     if ( args->count() != 1 ) args->usage();
 
     TestLibSyndication* tester = new TestLibSyndication(args->arg( 0 ));
+    Q_UNUSED(tester)
 
     return app.exec();
 }
